% setfocus -- set input focus.
%
%      setfocus(h)
%
% Set the input focus on Handle Graphics object H.

%% setfocus.m --- the preceding comment is the documentation string.

% Copyright (C) 2005--2008 Ralph Schleicher

% Author: Ralph Schleicher <rs@ralph-schleicher.de>
% Time-stamp: <2008-07-29 10:37:20 CEST>

% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License as
% published by the Free Software Foundation; either version 2,
% or (at your option) any later version.

% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

% You should have received a copy of the GNU General Public License
% along with this program; see the file COPYING.  If not, write to
% the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
% Boston, MA 02111-1307, USA.

% As a special exception, Ralph Schleicher gives permission to link
% the code of this program with MATLAB from The Mathworks, Inc. (or
% with modified versions of MATLAB that use the same license as
% MATLAB), and distribute linked combinations including the two.
% You must obey the GNU General Public License in all respects for
% all of the code used other than with MATLAB.  If you modify this
% file, you may extend this exception to your version of the file,
% but you are not obligated to do so.  If you do not wish to do so,
% delete this exception statement from your version.

%% Commentary:

% This is an improved version of Martin Weber's 'setfocus' command
% from Matlab Central.

%% Code:

% Program entry point.
function setfocus(h)

  if nargin < 1
    error(nargchk(1, 1, nargin));
  end

  if numel(h) ~= 1
    error('Invalid argument');
  end

  % Get figure handle.
  fig = get(h, 'Parent');
  if isempty(fig)
    return;
  elseif fig == 0
    fig = h;
  end

  % Raise figure.
  figure(fig);

  % Determine the focus point.
  units = get(fig, 'Units');
  set(fig, 'Units', 'pixels');
  fig_pos = get(fig, 'Position');
  set(fig, 'Units', units);
  % Lower left corner of the figure window.
  point = fig_pos(1:2);
  if fig ~= h
    units = get(h, 'Units');
    set(h, 'Units', 'pixels');
    obj_pos = get(h, 'Position');
    set(h, 'Units', units);
    % Upper left corner of the Handle Graphics object.
    point = point + obj_pos(1:2) + [1 obj_pos(4)] - 2;
    if ispc
      % Get off by 2.
      point = point + [2 -2];
    end
  end

  % Save call-back handler.
  try
    callback = get(h, 'Callback');
    if ~ isempty(callback)
      set(h, 'Callback', '');
    end
  catch
    callback = [];
  end

  % Save pointer location.
  mark = get(0, 'PointerLocation');

  % Click on the focus point.
  units = get(0, 'Units');
  set(0, 'Units', 'pixels');
  set(0, 'PointerLocation', point);
  set(0, 'Units', units);

  try
    drawnow;
    mouseclick;
  end

  % Restore pointer location.
  set(0, 'PointerLocation', mark);

  % Restore call-back handler.
  if ~ isempty(callback)
    set(h, 'Callback', callback);
  end

%% setfocus.m ends here
